==> conc-0.4.tar.gz: conc-0.4/src/console.c <==
/*
 This file is part of conc.

    Conc is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Conc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with conc; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <glib.h>
#include <popt.h>
#include "conc.h"

#define ESCAPE 1
#define USAGE "CONSOLE"
#define HELP "Run '%s --help' to see a full list of command line arguments\n"

GMainLoop *gml;

gchar *id;
gchar *identity;
gboolean escape=FALSE;
struct termios orig_tio;
ConcChannel *server;

#define LIST "l"
#define MESSAGE "Connected. ^A-q to quit. ^A-b to send break.\r\n"

gboolean
readlocal(GIOChannel *chan, GIOCondition cond, gpointer data)
{
  gchar c;
  gchar *s;
  int i;

  g_io_channel_read(chan,&c,1,&i);
  if (c==ESCAPE && !escape) {
    escape=TRUE;
    return TRUE;
  }
  if (escape) {
    switch (c) {
      case 'q': /*quit*/
        g_main_quit(gml);
        return FALSE;
      case 'b': /*send break*/
        s=g_strdup_printf("b%s",id);
        conc_channel_write(server,s);
        g_free(s);
        return TRUE;
    }
  }
  switch (c) {
    case 0:
      s=g_strdup_printf("z%s",id);
      break;
    case '\n':
      s=g_strdup_printf("T%s",id);
      break;
    default:
      s=g_strdup_printf("t%s%c",id,c);
      break;
  }
  conc_channel_write(server,s);
  g_free(s);
  return TRUE;
}

void
readremote(ConcChannel *chan, gchar *line, gint command_id, gpointer data)
{
  GIOChannel *local=(GIOChannel *)data;
  gint bytes;
  gchar x=0;

  switch (line[0]) {
      case 'I': /* identify - requires response */
        conc_channel_respond(chan,identity,command_id);
        break;
      case 'z':
        x=0;
        g_io_channel_write(local,&x,1,&bytes);
        break;
      case 'T':
        x='\n';
        g_io_channel_write(local,&x,1,&bytes);
        break;
      case 't':
        g_io_channel_write(local,&line[3],1,&bytes);
        break;
    }
  g_free(line);
}

void
got_list(ConcChannel *chan, gchar *l, gpointer cd, gpointer d)
{
  gint i;
  gchar **x;
  gchar *console=(gchar *)cd;

  x=g_strsplit(&l[1],":",0);
  g_free(l);
  for (i=0;x[i];i++) {
    if (!strcmp(&x[i][2],console)) break;
  }
  if (!x[i]) g_error("no console named %s",console);
  id=g_strdup_printf("%c%c",x[i][0],x[i][1]);
  g_strfreev(x);
  l=g_strdup_printf("rt%s",id);
  conc_channel_write(server,l);
  g_free(l);
}

void
server_closed(ConcChannel *chan, gpointer data)
{
  g_main_quit(gml);
}

int
main(int argc, const char *argv[])
{
  gint sock,i;
  struct sockaddr_in sa;
  struct hostent *he;
  GIOChannel *in;
  struct termios tio;
  gchar *server_name=NULL;
  gchar *console=NULL;
  gchar buf[256];
  struct poptOption options[] = {
    { "usage", '\0', POPT_ARG_NONE, NULL, 1, "Display brief usage message", NULL },
    { "help", '?', POPT_ARG_NONE, NULL, 2, "Show this help message", NULL },
    { "server", 's', POPT_ARG_STRING, &server_name, 0, "Server to use [default=localhost]", "SERVER"},
    { NULL, '\0', 0, NULL, 0 }
  };
  poptContext con;

  gml=g_main_new(FALSE);
  con=poptGetContext("console",argc,argv,options,0);
  poptSetOtherOptionHelp(con,USAGE);
  while ((i=poptGetNextOpt(con))>0) {
    switch (i) {
      case 1: /*usage*/
        poptPrintUsage(con,stdout,0);
        exit(0);
      case 2: /*help*/
        poptPrintHelp(con,stdout,0);
        exit(0);
    }
  }
  if (i<-1) {
    fprintf(stderr,"%s: %s\n", poptBadOption(con,POPT_BADOPTION_NOALIAS),
            poptStrerror(i));
    poptPrintUsage(con,stdout,0);
    fprintf(stderr,HELP,poptGetInvocationName(con));
    exit(1);
  }
  if (!server_name) server_name="localhost";
  console=(gchar *)poptGetArg(con);
  if (!console || poptPeekArg(con)) {
    poptPrintUsage(con,stdout,0);
    fprintf(stderr,HELP,poptGetInvocationName(con));
    exit(1);
  }
  sock=socket(AF_INET,SOCK_STREAM,0);
  if (sock==-1) {
    g_error("cannot create socket (%d)",errno);
  }
  if ((he=gethostbyname(server_name))==NULL) {
    g_error("cannot find server %s",server_name);
  }
  bzero(&sa,sizeof sa);
  bcopy(he->h_addr_list[0],(char *)&sa.sin_addr,he->h_length);
  sa.sin_family=AF_INET;
  sa.sin_port=htons(PORT);
  if (connect(sock,(struct sockaddr *)&sa,sizeof sa)==-1) {
    g_error("cannot connect to server %s (%d)",server_name,errno);
  }
  identity=conc_identity();
  if (tcgetattr(0,&tio)<0) g_error("cannot get local tty attributes");
  orig_tio=tio;
  tio.c_lflag &= ~(ECHO|ECHOE);
  if (tcsetattr(0,TCSANOW,&tio)<0) g_error("cannot set local tty attributes");
  g_print("Password: ");
  fgets(buf,256,stdin); g_print("\n");
  tcsetattr(0,TCSANOW,&orig_tio);
  for (i=0;buf[i];i++) if (buf[i]=='\n') buf[i]='\0';
  in=g_io_channel_unix_new(0);
  server=conc_channel_new(sock,sock,server_closed,readremote,in,buf);
  if (!server) g_error("bad password");
  for (i=0;buf[i];i++) buf[i]='*';
  conc_channel_command(server,LIST,got_list,console);
  cfmakeraw(&tio);
  tio.c_cflag |= HUPCL|CLOCAL|CREAD|CS8;
  tio.c_lflag |= NOFLSH;
  tio.c_lflag &= ~(ICANON|TOSTOP|ECHO|ECHOE|ECHOK|ECHONL|IEXTEN|ISIG);
  tio.c_iflag &= ~(IGNBRK|BRKINT|IGNPAR|PARMRK|INPCK|ISTRIP
                    |INLCR|IGNCR|ICRNL|IXON|IXOFF);
  tio.c_oflag &= ~OPOST;
  tio.c_cc[VMIN] = 0;
  tio.c_cc[VTIME] = 0;
  if (tcsetattr(0,TCSANOW,&tio)<0) g_error("cannot set local tty attributes");
  g_io_add_watch(in,G_IO_IN,readlocal,server);
  g_io_channel_write(in,MESSAGE,strlen(MESSAGE),&i);
  g_main_run(gml);
  tcsetattr(0,TCSANOW,&orig_tio);
  exit(0);
}

==> conc-0.5.tar.gz: conc-0.5/src/console.c <==
/*
 This file is part of conc.

    Conc is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Conc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with conc; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <glib.h>
#include <popt.h>
#include "conc.h"

#define ESCAPE 1
#define USAGE "CONSOLE"
#define HELP "Run '%s --help' to see a full list of command line arguments\n"

GMainLoop *gml;

gchar *id;
gchar *identity;
gboolean escape=FALSE;
struct termios orig_tio;
ConcChannel *server;

#define LIST "l"
#define MESSAGE "Connected. ^A-q to quit. ^A-b to send break.\r\n"

gboolean
readlocal(GIOChannel *chan, GIOCondition cond, gpointer data)
{
  gchar c;
  gchar *s;
  int i;

  g_io_channel_read(chan,&c,1,&i);
  if (c==ESCAPE && !escape) {
    escape=TRUE;
    return TRUE;
  }
  if (escape) {
    switch (c) {
      case 'q': /*quit*/
        g_main_quit(gml);
        return FALSE;
      case 'b': /*send break*/
        s=g_strdup_printf("b%s",id);
        conc_channel_write(server,s);
        g_free(s);
        return TRUE;
    }
  }
  switch (c) {
    case 0:
      s=g_strdup_printf("z%s",id);
      break;
    case '\n':
      s=g_strdup_printf("T%s",id);
      break;
    default:
      s=g_strdup_printf("t%s%c",id,c);
      break;
  }
  conc_channel_write(server,s);
  g_free(s);
  return TRUE;
}

void
readremote(ConcChannel *chan, gchar *line, gint command_id, gpointer data)
{
  GIOChannel *local=(GIOChannel *)data;
  gint bytes;
  gchar x=0;

  switch (line[0]) {
      case 'I': /* identify - requires response */
        conc_channel_respond(chan,identity,command_id);
        break;
      case 'z':
        x=0;
        g_io_channel_write(local,&x,1,&bytes);
        break;
      case 'T':
        x='\n';
        g_io_channel_write(local,&x,1,&bytes);
        break;
      case 't':
        g_io_channel_write(local,&line[3],1,&bytes);
        break;
    }
  g_free(line);
}

void
got_list(ConcChannel *chan, gchar *l, gpointer cd, gpointer d)
{
  gint i;
  gchar **x;
  gchar *console=(gchar *)cd;

  x=g_strsplit(&l[1],":",0);
  g_free(l);
  for (i=0;x[i];i++) {
    if (!strcmp(&x[i][2],console)) break;
  }
  if (!x[i]) g_error("no console named %s",console);
  id=g_strdup_printf("%c%c",x[i][0],x[i][1]);
  g_strfreev(x);
  l=g_strdup_printf("rt%s",id);
  conc_channel_write(server,l);
  g_free(l);
}

void
server_closed(ConcChannel *chan, gpointer data)
{
  g_main_quit(gml);
}

int
main(int argc, const char *argv[])
{
  gint sock,i;
  struct sockaddr_in sa;
  struct hostent *he;
  GIOChannel *in;
  struct termios tio;
  gchar *server_name=NULL;
  gchar *console=NULL;
  gchar buf[256];
  struct poptOption options[] = {
    { "usage", '\0', POPT_ARG_NONE, NULL, 1, "Display brief usage message", NULL },
    { "help", '?', POPT_ARG_NONE, NULL, 2, "Show this help message", NULL },
    { "server", 's', POPT_ARG_STRING, &server_name, 0, "Server to use [default=localhost]", "SERVER"},
    { NULL, '\0', 0, NULL, 0 }
  };
  poptContext con;

  gml=g_main_new(FALSE);
  con=poptGetContext("console",argc,argv,options,0);
  poptSetOtherOptionHelp(con,USAGE);
  while ((i=poptGetNextOpt(con))>0) {
    switch (i) {
      case 1: /*usage*/
        poptPrintUsage(con,stdout,0);
        exit(0);
      case 2: /*help*/
        poptPrintHelp(con,stdout,0);
        exit(0);
    }
  }
  if (i<-1) {
    fprintf(stderr,"%s: %s\n", poptBadOption(con,POPT_BADOPTION_NOALIAS),
            poptStrerror(i));
    poptPrintUsage(con,stdout,0);
    fprintf(stderr,HELP,poptGetInvocationName(con));
    exit(1);
  }
  if (!server_name) server_name="localhost";
  console=(gchar *)poptGetArg(con);
  if (!console || poptPeekArg(con)) {
    poptPrintUsage(con,stdout,0);
    fprintf(stderr,HELP,poptGetInvocationName(con));
    exit(1);
  }
  sock=socket(AF_INET,SOCK_STREAM,0);
  if (sock==-1) {
    g_error("cannot create socket (%d)",errno);
  }
  if ((he=gethostbyname(server_name))==NULL) {
    g_error("cannot find server %s",server_name);
  }
  bzero(&sa,sizeof sa);
  bcopy(he->h_addr_list[0],(char *)&sa.sin_addr,he->h_length);
  sa.sin_family=AF_INET;
  sa.sin_port=htons(CONC_PORT);
  if (connect(sock,(struct sockaddr *)&sa,sizeof sa)==-1) {
    g_error("cannot connect to server %s (%d)",server_name,errno);
  }
  identity=conc_identity();
  if (tcgetattr(0,&tio)<0) g_error("cannot get local tty attributes");
  orig_tio=tio;
  tio.c_lflag &= ~(ECHO|ECHOE);
  if (tcsetattr(0,TCSANOW,&tio)<0) g_error("cannot set local tty attributes");
  g_print("Password: ");
  fgets(buf,256,stdin); g_print("\n");
  tcsetattr(0,TCSANOW,&orig_tio);
  for (i=0;buf[i];i++) if (buf[i]=='\n') buf[i]='\0';
  in=g_io_channel_unix_new(0);
  server=conc_channel_new(sock,sock,server_closed,readremote,in,buf);
  if (!server) g_error("bad password");
  for (i=0;buf[i];i++) buf[i]='*';
  conc_channel_command(server,LIST,got_list,console);
  cfmakeraw(&tio);
  tio.c_cflag |= HUPCL|CLOCAL|CREAD|CS8;
  tio.c_lflag |= NOFLSH;
  tio.c_lflag &= ~(ICANON|TOSTOP|ECHO|ECHOE|ECHOK|ECHONL|IEXTEN|ISIG);
  tio.c_iflag &= ~(IGNBRK|BRKINT|IGNPAR|PARMRK|INPCK|ISTRIP
                    |INLCR|IGNCR|ICRNL|IXON|IXOFF);
  tio.c_oflag &= ~OPOST;
  tio.c_cc[VMIN] = 0;
  tio.c_cc[VTIME] = 0;
  if (tcsetattr(0,TCSANOW,&tio)<0) g_error("cannot set local tty attributes");
  g_io_add_watch(in,G_IO_IN,readlocal,server);
  g_io_channel_write(in,MESSAGE,strlen(MESSAGE),&i);
  g_main_run(gml);
  tcsetattr(0,TCSANOW,&orig_tio);
  exit(0);
}
